@@ -490,26 +490,47 @@ print "OK\n\n";
 if ($config{crossconf}) {
     build::auto::detect_cross(\%config, \%defaults);
     build::probe::static_inline_cross(\%config, \%defaults);
+    build::probe::thread_local_cross(\%config, \%defaults);
     build::probe::unaligned_access_cross(\%config, \%defaults);
     build::probe::ptr_size_cross(\%config, \%defaults);
 }
 else {
     build::auto::detect_native(\%config, \%defaults);
     build::probe::static_inline_native(\%config, \%defaults);
+    build::probe::thread_local_native(\%config, \%defaults);
     build::probe::unaligned_access(\%config, \%defaults);
     build::probe::ptr_size_native(\%config, \%defaults);
 }
 
 
+my $archname = $Config{archname};
 if ($args{'jit'}) {
     if ($config{ptr_size} != 8) {
         print "JIT isn't supported on platforms with $config{ptr_size} byte pointers.\n";
-    } elsif ($Config{archname} =~ m/^x86_64|^amd64|^darwin(-thread)?(-multi)?-2level/) {
+    } elsif ($archname =~ m/^x86_64|^amd64/) {
         $config{jit_obj}      = '$(JIT_OBJECTS) $(JIT_ARCH_X64)';
         $config{dasm_flags}   = '-D POSIX=1';
         $config{jit_arch}     = 'MVM_JIT_ARCH_X64';
         $config{jit_platform} = 'MVM_JIT_PLATFORM_POSIX';
-    } elsif ($Config{archname} =~ /^MSWin32-x64/) {
+    } elsif ($archname =~ m/^darwin(-thread)?(-multi)?-2level/) {
+        hardfail("Missing /usr/bin/arch") if !-x '/usr/bin/arch';
+        my $arch = `/usr/bin/arch`;
+        chomp $arch;
+        if ($arch ne 'arm64') {
+            $config{jit_obj}      = '$(JIT_OBJECTS) $(JIT_ARCH_X64)';
+            $config{dasm_flags}   = '-D POSIX=1';
+            $config{jit_arch}     = 'MVM_JIT_ARCH_X64';
+            $config{jit_platform} = 'MVM_JIT_PLATFORM_POSIX';
+        } else {
+            print "JIT isn't supported on $Config{archname} ARM64 yet.\n";
+# future support of ARM64 JITting
+#            $config{jit_obj}      = '$(JIT_OBJECTS) $(JIT_ARCH_)';
+#            $config{dasm_flags}   = '-D POSIX=1';
+#            $config{jit_arch}     = 'MVM_JIT_ARCH_ARM64';
+#            $config{jit_platform} = 'MVM_JIT_PLATFORM_POSIX';
+        }
+
+    } elsif ($archname =~ /^MSWin32-x64/) {
         $config{jit_obj}      = '$(JIT_OBJECTS) $(JIT_ARCH_X64)';
         $config{dasm_flags}   = '-D WIN32=1';
         $config{jit_arch}     = 'MVM_JIT_ARCH_X64';
