@@ -740,23 +740,18 @@ static const MVMContainerConfigurer NativeRefContainerConfigurer = {
 /* Adds a container configurer to the registry. */
 void MVM_6model_add_container_config(MVMThreadContext *tc, MVMString *name,
         const MVMContainerConfigurer *configurer) {
-    MVMContainerRegistry *entry;
+
+    if (!MVM_str_hash_key_is_valid(tc, name)) {
+        MVM_str_hash_key_throw_invalid(tc, name);
+    }
 
     uv_mutex_lock(&tc->instance->mutex_container_registry);
 
-    MVM_HASH_GET(tc, tc->instance->container_registry, name, entry);
+    MVMContainerRegistry *entry = MVM_str_hash_lvalue_fetch_nocheck(tc, &tc->instance->container_registry, name);
 
-    if (!entry) {
-        entry = MVM_malloc(sizeof(MVMContainerRegistry));
-        MVM_HASH_BIND_FREE(tc, tc->instance->container_registry, name, entry, {
-            MVM_free(entry);
-        });
-        entry->name = name;
-        entry->configurer  = configurer;
-        MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->name,
-            "Container configuration name");
-        MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->hash_handle.key,
-            "Container configuration hash key");
+    if (!entry->hash_handle.key) {
+        entry->configurer      = configurer;
+        entry->hash_handle.key = name;
     }
 
     uv_mutex_unlock(&tc->instance->mutex_container_registry);
@@ -764,11 +759,15 @@ void MVM_6model_add_container_config(MVMThreadContext *tc, MVMString *name,
 
 /* Gets a container configurer from the registry. */
 const MVMContainerConfigurer * MVM_6model_get_container_config(MVMThreadContext *tc, MVMString *name) {
-    MVMContainerRegistry *entry;
+
+    if (!MVM_str_hash_key_is_valid(tc, name)) {
+        MVM_str_hash_key_throw_invalid(tc, name);
+    }
+
     uv_mutex_lock(&tc->instance->mutex_container_registry);
-    MVM_HASH_GET(tc, tc->instance->container_registry, name, entry);
+    MVMContainerRegistry *entry = MVM_str_hash_fetch_nocheck(tc, &tc->instance->container_registry, name);
     uv_mutex_unlock(&tc->instance->mutex_container_registry);
-    return entry != NULL ? entry->configurer : NULL;
+    return entry ? entry->configurer : NULL;
 }
 
 /* Does initial setup work of the container registry, including registering
