@@ -167,7 +167,7 @@ void MVM_code_location_out(MVMThreadContext *tc, MVMObject *code,
         MVMCodeBody          *body = &((MVMCode*)code)->body;
         MVMBytecodeAnnotation *ann = MVM_bytecode_resolve_annotation(tc, &body->sf->body, 0);
         MVMCompUnit            *cu = body->sf->body.cu;
-        MVMint32           str_idx = ann ? ann->filename_string_heap_index : 0;
+        MVMuint32          str_idx = ann ? ann->filename_string_heap_index : 0;
 
         *line_out = ann ? ann->line_number : 1;
         if (ann && str_idx < cu->body.num_strings) {
