@@ -450,9 +450,6 @@ MVMint64 MVM_coerce_simple_intify(MVMThreadContext *tc, MVMObject *obj) {
     }
 }
 
-/* concatenating with "" ensures that only literal strings are accepted as argument. */
-#define STR_WITH_LEN(str)  ("" str ""), (sizeof(str) - 1)
-
 MVMObject * MVM_radix(MVMThreadContext *tc, MVMint64 radix, MVMString *str, MVMint64 offset, MVMint64 flag) {
     MVMObject *result;
     MVMint64 zvalue = 0;
@@ -525,58 +522,3 @@ MVMObject * MVM_radix(MVMThreadContext *tc, MVMint64 radix, MVMString *str, MVMi
 
     return result;
 }
-
-
-void MVM_box_int(MVMThreadContext *tc, MVMint64 value, MVMObject *type,
-             MVMRegister * dst) {
-    MVMObject *box = MVM_intcache_get(tc, type, value);
-    if (box == 0) {
-        box = REPR(type)->allocate(tc, STABLE(type));
-        if (REPR(box)->initialize)
-            REPR(box)->initialize(tc, STABLE(box), box, OBJECT_BODY(box));
-        REPR(box)->box_funcs.set_int(tc, STABLE(box), box,
-                                     OBJECT_BODY(box), value);
-    }
-    dst->o = box;
-}
-
-void MVM_box_num(MVMThreadContext *tc, MVMnum64 value, MVMObject *type,
-                 MVMRegister * dst) {
-    MVMObject *box = REPR(type)->allocate(tc, STABLE(type));
-    if (REPR(box)->initialize)
-        REPR(box)->initialize(tc, STABLE(box), box, OBJECT_BODY(box));
-    REPR(box)->box_funcs.set_num(tc, STABLE(box), box,
-                                 OBJECT_BODY(box), value);
-    dst->o = box;
-
-}
-
-MVMString * MVM_unbox_str(MVMThreadContext *tc, MVMObject *obj) {
-    if (!IS_CONCRETE(obj))
-        MVM_exception_throw_adhoc(tc, "Cannot unbox a type object (%s) to a str.",
-            MVM_6model_get_debug_name(tc, obj));
-    return REPR(obj)->box_funcs.get_str(tc,
-        STABLE(obj), obj, OBJECT_BODY(obj));
-}
-
-void MVM_box_str(MVMThreadContext *tc, MVMString *value, MVMObject *type,
-                 MVMRegister *dst) {
-    MVMObject *box;
-    MVMROOT(tc, value, {
-        box = REPR(type)->allocate(tc, STABLE(type));
-        if (REPR(box)->initialize)
-            REPR(box)->initialize(tc, STABLE(box), box, OBJECT_BODY(box));
-        REPR(box)->box_funcs.set_str(tc, STABLE(box), box,
-                                     OBJECT_BODY(box), value);
-        dst->o = box;
-    });
-}
-
-void MVM_box_uint(MVMThreadContext *tc, MVMuint64 value, MVMObject *type,
-             MVMRegister * dst) {
-    MVMObject *box = REPR(type)->allocate(tc, STABLE(type));
-    if (REPR(box)->initialize)
-        REPR(box)->initialize(tc, STABLE(box), box, OBJECT_BODY(box));
-    REPR(box)->box_funcs.set_uint(tc, STABLE(box), box, OBJECT_BODY(box), value);
-    dst->o = box;
-}
