@@ -3,27 +3,21 @@
 MVMHLLConfig *MVM_hll_get_config_for(MVMThreadContext *tc, MVMString *name) {
     MVMHLLConfig *entry;
 
+    if (!MVM_str_hash_key_is_valid(tc, name)) {
+        MVM_str_hash_key_throw_invalid(tc, name);
+    }
+
     uv_mutex_lock(&tc->instance->mutex_hllconfigs);
 
     if (tc->instance->hll_compilee_depth) {
-        MVM_HASH_GET(tc, tc->instance->compilee_hll_configs, name, entry);
+        entry = MVM_fixkey_hash_lvalue_fetch_nocheck(tc, &tc->instance->compilee_hll_configs, name);
     }
     else {
-        MVM_HASH_GET(tc, tc->instance->compiler_hll_configs, name, entry);
+        entry = MVM_fixkey_hash_lvalue_fetch_nocheck(tc, &tc->instance->compiler_hll_configs, name);
     }
 
-    if (!entry) {
-        entry = MVM_calloc(1, sizeof(MVMHLLConfig));
-        if (tc->instance->hll_compilee_depth) {
-            MVM_HASH_BIND_FREE(tc, tc->instance->compilee_hll_configs, name, entry, {
-                MVM_free(entry);
-            });
-        }
-        else {
-            MVM_HASH_BIND_FREE(tc, tc->instance->compiler_hll_configs, name, entry, {
-                MVM_free(entry);
-            });
-        }
+    if (!entry->name) {
+        memset(entry, 0, sizeof(*entry));
         entry->name = name;
         entry->int_box_type = tc->instance->boot_types.BOOTInt;
         entry->num_box_type = tc->instance->boot_types.BOOTNum;
@@ -69,7 +63,6 @@ MVMHLLConfig *MVM_hll_get_config_for(MVMThreadContext *tc, MVMString *name) {
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->str_multidim_ref, "HLL str_multidim_ref");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->spesh_plugins, "HLL spesh plugins");
         MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->name, "HLL name");
-        MVM_gc_root_add_permanent_desc(tc, (MVMCollectable **)&entry->hash_handle.key, "HLL hash key");
     }
 
     uv_mutex_unlock(&tc->instance->mutex_hllconfigs);
