@@ -1,5 +1,4 @@
 #include "moar.h"
-#include <platform/threads.h>
 
 /* Temporary structure for passing data to thread start. */
 typedef struct {
@@ -76,6 +75,11 @@ static void start_thread(void *data) {
     /* Stash thread ID. */
     tc->thread_obj->body.native_thread_id = MVM_platform_thread_id();
 
+    /* Store this thread's thread context pointer, so that we can retrieve it
+     * in places where we can't pass it in to, such as the callback function
+     * used by qsort. */
+    MVM_set_running_threads_context(tc);
+
     /* Create a spesh log for this thread, unless it's just going to run C
      * code (and thus it's a VM internal worker). */
     if (REPR(tc->thread_obj->body.invokee)->ID != MVM_REPR_ID_MVMCFunction)
