@@ -236,7 +236,7 @@ static void on_write(uv_write_t *req, int status) {
     }
     MVM_repr_push_o(tc, t->body.queue, arr);
     MVM_io_eventloop_remove_active_work(tc, &(wi->work_idx));
-    MVM_free(wi->req);
+    MVM_free_null(wi->req);
 }
 
 /* Does setup work for an asynchronous write. */
@@ -269,8 +269,7 @@ static void write_setup(MVMThreadContext *tc, uv_loop_t *loop, MVMObject *async_
         /* Error; need to notify. */
         MVMROOT(tc, async_task, {
             MVMObject    *arr = MVM_repr_alloc_init(tc, tc->instance->boot_types.BOOTArray);
-            MVMAsyncTask *t   = (MVMAsyncTask *)async_task;
-            MVM_repr_push_o(tc, arr, t->body.schedulee);
+            MVM_repr_push_o(tc, arr, ((MVMAsyncTask *)async_task)->body.schedulee);
             MVM_repr_push_o(tc, arr, tc->instance->boot_types.BOOTInt);
             MVMROOT(tc, arr, {
                 MVMString *msg_str = MVM_string_ascii_decode_nt(tc,
@@ -285,12 +284,11 @@ static void write_setup(MVMThreadContext *tc, uv_loop_t *loop, MVMObject *async_
                     tc->instance->boot_types.BOOTStr, msg_str);
                 MVM_repr_push_o(tc, arr, msg_box);
             });
-            MVM_repr_push_o(tc, t->body.queue, arr);
+            MVM_repr_push_o(tc, ((MVMAsyncTask *)async_task)->body.queue, arr);
         });
 
         /* Cleanup handle. */
-        MVM_free(wi->req);
-        wi->req = NULL;
+        MVM_free_null(wi->req);
     }
 }
 
@@ -751,7 +749,7 @@ static void spawn_setup(MVMThreadContext *tc, uv_loop_t *loop, MVMObject *async_
     process_options.file        = si->prog;
     process_options.args        = si->args;
     process_options.cwd         = si->cwd;
-    process_options.flags       = UV_PROCESS_WINDOWS_HIDE;
+    process_options.flags       = UV_PROCESS_WINDOWS_HIDE | UV_PROCESS_WINDOWS_VERBATIM_ARGUMENTS;
     process_options.env         = si->env;
     process_options.stdio_count = 3;
     process_options.exit_cb     = async_spawn_on_exit;
@@ -947,8 +945,7 @@ static void spawn_gc_free(MVMThreadContext *tc, MVMObject *t, void *data) {
     if (data) {
         SpawnInfo *si = (SpawnInfo *)data;
         if (si->cwd) {
-            MVM_free(si->cwd);
-            si->cwd = NULL;
+            MVM_free_null(si->cwd);
         }
         if (si->env) {
             MVMuint32 i;
@@ -960,8 +957,7 @@ static void spawn_gc_free(MVMThreadContext *tc, MVMObject *t, void *data) {
             MVMuint32 i = 0;
             while (si->args[i])
                 MVM_free(si->args[i++]);
-            MVM_free(si->args);
-            si->args = NULL;
+            MVM_free_null(si->args);
         }
         MVM_free(si);
     }
