@@ -39,6 +39,9 @@
 #include <dyncall_callback.h>
 #endif
 
+/* needed in threadcontext.h */
+#include <platform/threads.h>
+
 /* forward declarations */
 #include "types.h"
 
@@ -96,6 +99,7 @@ typedef double   MVMnum64;
 #define HASH_DEBUG_ITER 0
 #define MVM_HASH_RANDOMIZE 1
 #define MVM_HASH_MAX_PROBE_DISTANCE 255
+#define MVM_HASH_INITIAL_BITS_IN_METADATA 5
 
 typedef MVMuint32 MVMHashNumItems;
 typedef MVMuint64 MVMHashv;
@@ -108,13 +112,13 @@ MVM_PUBLIC MVMint32 MVM_jit_support(void);
 #include "gc/debug.h"
 #include "gc/wb.h"
 #include "core/vector.h"
-#include "core/threadcontext.h"
 #include "core/exceptions.h"
 #include "core/str_hash_table.h"
 #include "core/fixkey_hash_table.h"
 #include "core/index_hash_table.h"
 #include "core/ptr_hash_table.h"
 #include "core/uni_hash_table.h"
+#include "core/threadcontext.h"
 #include "core/instance.h"
 #include "core/interp.h"
 #include "core/callsite.h"
