@@ -7,27 +7,6 @@ typedef struct {
     MVMProfileCallNode **list;
 } NodeWorklist;
 
-static void add_node(MVMThreadContext *tc, NodeWorklist *list, MVMProfileCallNode *node) {
-    if (list->items == list->alloc) {
-        /* Filled up the whole list. Make it bigger */
-        list->alloc *= 2;
-        list->list = MVM_realloc(list->list, list->alloc * sizeof(MVMProfileCallNode *));
-    }
-    /* Add at the end */
-    list->list[list->items] = node;
-    list->items++;
-}
-
-static MVMProfileCallNode *take_node(MVMThreadContext *tc, NodeWorklist *list) {
-    MVMProfileCallNode *result = NULL;
-    if (list->items == 0) {
-        MVM_panic(1, "profiler: tried to take a node from an empty node worklist");
-    }
-    list->items--;
-    result = list->list[list->items];
-    return result;
-}
-
 /* Adds an instruction to log an allocation. */
 static void add_allocation_logging_at_location(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB *bb, MVMSpeshIns *ins, MVMSpeshIns *location) {
     MVMSpeshIns *alloc_ins = MVM_spesh_alloc(tc, g, sizeof(MVMSpeshIns));
@@ -103,6 +82,8 @@ static void instrument_graph(MVMThreadContext *tc, MVMSpeshGraph *g) {
             case MVM_OP_invoke_o:
             case MVM_OP_param_rp_o:
             case MVM_OP_param_rn_o:
+            case MVM_OP_param_rn2_o:
+            case MVM_OP_param_rn2_s:
             case MVM_OP_param_sp:
             case MVM_OP_param_sn:
             case MVM_OP_newexception:
@@ -116,6 +97,7 @@ static void instrument_graph(MVMThreadContext *tc, MVMSpeshGraph *g) {
             case MVM_OP_sp_fastcreate:
             case MVM_OP_clone:
             case MVM_OP_box_i:
+            case MVM_OP_box_u:
             case MVM_OP_box_n:
             case MVM_OP_box_s:
             case MVM_OP_iter:
@@ -137,11 +119,116 @@ static void instrument_graph(MVMThreadContext *tc, MVMSpeshGraph *g) {
             case MVM_OP_lcm_I:
             case MVM_OP_expmod_I:
             case MVM_OP_rand_I:
+            case MVM_OP_base_I:
             case MVM_OP_coerce_nI:
             case MVM_OP_coerce_sI:
+            case MVM_OP_coerce_II:
             case MVM_OP_coerce_is:
+            case MVM_OP_coerce_us:
             case MVM_OP_coerce_ns:
             case MVM_OP_smrt_strify:
+            case MVM_OP_buffertocu:
+            case MVM_OP_serializetobuf:
+            case MVM_OP_getsignals:
+            case MVM_OP_decodersetlineseps:
+            case MVM_OP_decoderconfigure:
+            case MVM_OP_decoderaddbytes:
+            case MVM_OP_decodertakeallchars:
+            case MVM_OP_decodertakeavailablechars:
+            case MVM_OP_decodertakeline:
+            case MVM_OP_decodertakebytes:
+            case MVM_OP_decodertakechars:
+            case MVM_OP_decodertakecharseof:
+            case MVM_OP_decodeconf:
+            case MVM_OP_decoderepconf:
+            case MVM_OP_strfromname:
+            case MVM_OP_slice:
+            case MVM_OP_loadbytecode:
+            case MVM_OP_loadbytecodefh:
+            case MVM_OP_multidimref_i:
+            case MVM_OP_multidimref_n:
+            case MVM_OP_multidimref_s:
+            case MVM_OP_asyncudp:
+            case MVM_OP_asyncwritebytesto:
+            case MVM_OP_asyncwritebytes:
+            case MVM_OP_asyncreadbytes:
+            case MVM_OP_encoderep:
+            case MVM_OP_lc:
+            case MVM_OP_uc:
+            case MVM_OP_fc:
+            case MVM_OP_getcodelocation:
+            case MVM_OP_normalizecodes:
+            case MVM_OP_strtocodes:
+            case MVM_OP_strfromcodes:
+            case MVM_OP_readlink:
+            case MVM_OP_parameterizetype:
+            case MVM_OP_spawnprocasync:
+            case MVM_OP_execname:
+            case MVM_OP_getuniprop_str:
+            case MVM_OP_getuniname:
+            case MVM_OP_sp_add_I:
+            case MVM_OP_sp_sub_I:
+            case MVM_OP_sp_mul_I:
+            case MVM_OP_uname:
+            case MVM_OP_decodelocaltime:
+            case MVM_OP_indexingoptimized:
+            case MVM_OP_setdimensions:
+            case MVM_OP_dimensions:
+            case MVM_OP_watchfile:
+            case MVM_OP_timer:
+            case MVM_OP_ctx:
+            case MVM_OP_ctxouter:
+            case MVM_OP_ctxcaller:
+            case MVM_OP_ctxouterskipthunks:
+            case MVM_OP_ctxcallerskipthunks:
+            case MVM_OP_getlockcondvar:
+            case MVM_OP_gethostname:
+            case MVM_OP_backtrace:
+            case MVM_OP_backtracestrings:
+            case MVM_OP_replace:
+            case MVM_OP_capturenamedshash:
+            case MVM_OP_bitand_s:
+            case MVM_OP_bitor_s:
+            case MVM_OP_bitxor_s:
+            case MVM_OP_backendconfig:
+            case MVM_OP_findsym:
+            case MVM_OP_getenvhash:
+            case MVM_OP_clargs:
+            case MVM_OP_cwd:
+            case MVM_OP_newthread:
+            case MVM_OP_tryfindmeth:
+            case MVM_OP_tryfindmeth_s:
+            case MVM_OP_accept_sk:
+            case MVM_OP_bind_sk:
+            case MVM_OP_connect_sk:
+            case MVM_OP_socket:
+            case MVM_OP_open_fh:
+            case MVM_OP_read_dir:
+            case MVM_OP_open_dir:
+            case MVM_OP_compunitcodes:
+            case MVM_OP_newmixintype:
+            case MVM_OP_scgetdesc:
+            case MVM_OP_popcompsc:
+            case MVM_OP_createsc:
+            case MVM_OP_sha1:
+            case MVM_OP_freshcoderef:
+            case MVM_OP_gethllsym:
+            case MVM_OP_getcurhllsym:
+            case MVM_OP_settypecache:
+            case MVM_OP_setmethcache:
+            case MVM_OP_newtype:
+            case MVM_OP_nfafromstatelist:
+            case MVM_OP_nfarunproto:
+            case MVM_OP_flip:
+            case MVM_OP_escape:
+            case MVM_OP_chr:
+            case MVM_OP_join:
+            case MVM_OP_split:
+            case MVM_OP_concat_s:
+            case MVM_OP_repeat_s:
+            case MVM_OP_substr_s:
+            case MVM_OP_multicacheadd:
+            case MVM_OP_radix:
             case MVM_OP_radix_I: {
                 add_allocation_logging(tc, g, bb, ins);
                 break;
@@ -177,7 +264,15 @@ static void instrument_graph(MVMThreadContext *tc, MVMSpeshGraph *g) {
                 break;
             }
             case MVM_OP_getlexref_i:
+            case MVM_OP_getlexref_i8:
+            case MVM_OP_getlexref_i16:
+            case MVM_OP_getlexref_i32:
+            case MVM_OP_getlexref_u:
+            case MVM_OP_getlexref_u8:
+            case MVM_OP_getlexref_u16:
+            case MVM_OP_getlexref_u32:
             case MVM_OP_getlexref_n:
+            case MVM_OP_getlexref_n32:
             case MVM_OP_getlexref_s:
             case MVM_OP_getlexref_ni:
             case MVM_OP_getlexref_nn:
@@ -192,6 +287,7 @@ static void instrument_graph(MVMThreadContext *tc, MVMSpeshGraph *g) {
             case MVM_OP_getattrsref_n:
             case MVM_OP_getattrsref_s:
             case MVM_OP_nativecallcast:
+            case MVM_OP_nativecallglobal:
                 add_allocation_logging(tc, g, bb, ins);
                 break;
             case MVM_OP_nativecallinvoke:
@@ -249,9 +345,8 @@ void MVM_profile_instrument(MVMThreadContext *tc, MVMStaticFrame *sf) {
         sf->body.handlers      = sf->body.instrumentation->instrumented_handlers;
         sf->body.bytecode_size = sf->body.instrumentation->instrumented_bytecode_size;
 
-        /* Throw away any argument guard so we'll never resolve prior
-         * specializations again. */
-        MVM_spesh_arg_guard_discard(tc, sf);
+        /* Throw away any existing specializations. */
+        MVM_spesh_candidate_discard_existing(tc, sf);
     }
 }
 
@@ -350,7 +445,7 @@ typedef struct {
 } ProfTcPdsStruct;
 
 static MVMObject * insert_if_not_exists(MVMThreadContext *tc, ProfDumpStrs *pds, MVMObject *storage, MVMint64 key) {
-    MVMint64 index;
+    MVMuint64 index;
     MVMObject *result;
     MVMObject *type_info_hash;
 
@@ -447,23 +542,24 @@ static MVMObject * dump_call_graph_node(MVMThreadContext *tc, ProfDumpStrs *pds,
     MVMuint64 absolute_start_time;
 
     /* Let's see if we're dealing with a native call or a regular moar call */
-    if (pcn->sf) {
+    if (tc->prof_data->staticframe_array[pcn->sf_idx]) {
+        MVMStaticFrame *sf = tc->prof_data->staticframe_array[pcn->sf_idx];
         /* Try to resolve the code filename and line number. */
         MVMBytecodeAnnotation *annot = MVM_bytecode_resolve_annotation(tc,
-            &(pcn->sf->body), 0);
-        MVMint32 fshi = annot ? (MVMint32)annot->filename_string_heap_index : -1;
+            &(sf->body), 0);
+        MVMuint32 fshi = annot ? (MVMint32)annot->filename_string_heap_index : 0;
 
         /* Add name of code object. */
         MVM_repr_bind_key_o(tc, node_hash, pds->name,
-            box_s(tc, pcn->sf->body.name));
+            box_s(tc, sf->body.name));
 
         /* Add line number and file name. */
-        if (fshi >= 0 && fshi < pcn->sf->body.cu->body.num_strings)
+        if (annot && fshi < sf->body.cu->body.num_strings)
             MVM_repr_bind_key_o(tc, node_hash, pds->file,
-                box_s(tc, MVM_cu_string(tc, pcn->sf->body.cu, fshi)));
-        else if (pcn->sf->body.cu->body.filename)
+                box_s(tc, MVM_cu_string(tc, sf->body.cu, fshi)));
+        else if (sf->body.cu->body.filename)
             MVM_repr_bind_key_o(tc, node_hash, pds->file,
-                box_s(tc, pcn->sf->body.cu->body.filename));
+                box_s(tc, sf->body.cu->body.filename));
         else
             MVM_repr_bind_key_o(tc, node_hash, pds->file,
                 box_s(tc, tc->instance->str_consts.empty));
@@ -473,7 +569,7 @@ static MVMObject * dump_call_graph_node(MVMThreadContext *tc, ProfDumpStrs *pds,
 
         /* Use static frame memory address to get a unique ID. */
         MVM_repr_bind_key_o(tc, node_hash, pds->id,
-            box_i(tc, (MVMint64)(uintptr_t)pcn->sf));
+            box_i(tc, (MVMint64)(uintptr_t)sf));
     } else if (pcn->native_target_name) {
         MVMString *function_name_string =
             MVM_string_utf8_c8_decode(tc, tc->instance->VMString,
@@ -548,7 +644,7 @@ static MVMObject * dump_call_graph_node(MVMThreadContext *tc, ProfDumpStrs *pds,
             MVMObject *alloc_info = new_hash(tc);
             MVMProfileAllocationCount *alloc = &pcn->alloc[i];
 
-            MVMObject *type       = pcn->alloc[i].type;
+            MVMObject *type = tc->prof_data->type_array[pcn->alloc[i].type_idx];
 
             add_type_to_types_array(tc, pds, type, types_array);
 
@@ -815,15 +911,6 @@ MVMObject * MVM_profile_instrumented_end(MVMThreadContext *tc) {
 }
 
 
-/* Marks objects held in the profiling graph. */
-static void mark_call_graph_node(MVMThreadContext *tc, MVMProfileCallNode *node, NodeWorklist *nodelist, MVMGCWorklist *worklist) {
-    MVMuint32 i;
-    MVM_gc_worklist_add(tc, worklist, &(node->sf));
-    for (i = 0; i < node->num_alloc; i++)
-        MVM_gc_worklist_add(tc, worklist, &(node->alloc[i].type));
-    for (i = 0; i < node->num_succ; i++)
-        add_node(tc, nodelist, node->succ[i]);
-}
 static void mark_gc_entries(MVMThreadContext *tc, MVMProfileThreadData *ptd, MVMGCWorklist *worklist) {
     MVMuint32 gci;
     for (gci = 0; gci < ptd->num_gcs; gci++) {
@@ -836,41 +923,33 @@ static void mark_gc_entries(MVMThreadContext *tc, MVMProfileThreadData *ptd, MVM
 }
 void MVM_profile_instrumented_mark_data(MVMThreadContext *tc, MVMGCWorklist *worklist) {
     if (tc->prof_data) {
-        /* Allocate our worklist on the stack. */
-        NodeWorklist nodelist;
-        nodelist.items = 0;
-        nodelist.alloc = 256;
-        nodelist.list = MVM_malloc(nodelist.alloc * sizeof(MVMProfileCallNode *));
+        MVMProfileThreadData *ptd = tc->prof_data;
+        MVMuint32 index;
 
-        add_node(tc, &nodelist, tc->prof_data->call_graph);
-
-        while (nodelist.items) {
-            MVMProfileCallNode *node = take_node(tc, &nodelist);
-            if (node)
-                mark_call_graph_node(tc, node, &nodelist, worklist);
-        }
+        for (index = 0; index < MVM_VECTOR_ELEMS(ptd->staticframe_array); index++)
+            MVM_gc_worklist_add(tc, worklist, &(ptd->staticframe_array[index]));
+        for (index = 0; index < MVM_VECTOR_ELEMS(ptd->type_array); index++)
+            MVM_gc_worklist_add(tc, worklist, &(ptd->type_array[index]));
 
         MVM_gc_worklist_add(tc, worklist, &(tc->prof_data->collected_data));
 
         mark_gc_entries(tc, tc->prof_data, worklist);
-
-        MVM_free(nodelist.list);
     }
 }
 
 static void dump_callgraph_node(MVMThreadContext *tc, MVMProfileCallNode *n, MVMuint16 depth) {
     MVMuint16 dc = depth;
     MVMuint32 idx;
-    char *name;
+    char *name = NULL;
 
     for (dc = depth; dc > 0; dc--) {
         fputc(' ', stderr);
     }
 
-    if (n->sf)
-        name = MVM_string_utf8_encode_C_string(tc, n->sf->body.name);
+    if (tc->prof_data->staticframe_array[n->sf_idx])
+        name = MVM_string_utf8_encode_C_string(tc, tc->prof_data->staticframe_array[n->sf_idx]->body.name);
 
-    fprintf(stderr, "+ [%3d] %s\n", n->num_succ, name);
+    fprintf(stderr, "+ [%3d] %s\n", n->num_succ, name ? name : "(unknown)");
     MVM_free(name);
 
     for (idx = 0; idx < n->num_succ; idx++) {
