@@ -28,7 +28,15 @@ void MVM_spesh_manipulate_delete_ins(MVMThreadContext *tc, MVMSpeshGraph *g,
     while (ins->annotations) {
         MVMSpeshAnn *ann      = ins->annotations;
         MVMSpeshAnn *ann_next = ann->next;
-        switch (ann->type) {
+        /* Special case: we make fake entires into the handler table for
+         * a handler that covers the whole inline, so we can proprerly
+         * process inline boundaries. Those need to move line an inline
+         * end annotation. */
+        int tweaked_type = ann->type == MVM_SPESH_ANN_FH_END &&
+                g->handlers[ann->data.frame_handler_index].category_mask == MVM_EX_INLINE_BOUNDARY
+            ? MVM_SPESH_ANN_INLINE_END
+            : ann->type;
+        switch (tweaked_type) {
             case MVM_SPESH_ANN_FH_START:
             case MVM_SPESH_ANN_FH_GOTO:
             case MVM_SPESH_ANN_INLINE_START:
@@ -65,7 +73,9 @@ void MVM_spesh_manipulate_delete_ins(MVMThreadContext *tc, MVMSpeshGraph *g,
                 /* This moves to the previous instruction, but we need to put
                  * it on the end of the list, so the earlier deopt point will
                  * win when searching for deopt points. Otherwise, we can
-                 * deopt to a later place than we should have. */
+                 * deopt to a later place than we should have. Also, we should
+                 * never move a later deopt instruction onto something with a
+                 * deopt all or an inline, otherwise it can confuse uninlining. */
                 if (!prev) {
                     MVMSpeshBB *prev_bb = MVM_spesh_graph_linear_prev(tc, g, bb);
                     while (prev_bb && !prev_bb->last_ins)
@@ -75,13 +85,24 @@ void MVM_spesh_manipulate_delete_ins(MVMThreadContext *tc, MVMSpeshGraph *g,
                 }
                 if (prev) {
                     MVMSpeshAnn *append_to = prev->annotations;
-                    while (append_to && append_to->next)
+                    MVMint32 conflict = 0;
+                    while (append_to) {
+                        if (append_to->type == MVM_SPESH_ANN_DEOPT_ALL_INS ||
+                                append_to->type == MVM_SPESH_ANN_DEOPT_INLINE) {
+                            conflict = 1;
+                            break;
+                        }
+                        if (!append_to->next)
+                            break;
                         append_to = append_to->next;
-                    if (append_to)
-                        append_to->next = ann;
-                    else
-                        prev->annotations = ann;
-                    ann->next = NULL;
+                    }
+                    if (!conflict) {
+                        if (append_to)
+                            append_to->next = ann;
+                        else
+                            prev->annotations = ann;
+                        ann->next = NULL;
+                    }
                 }
                 break;
         }
@@ -121,7 +142,7 @@ void MVM_spesh_manipulate_cleanup_ins_deps(MVMThreadContext *tc, MVMSpeshGraph *
     }
 }
 
-/* Inserts an instruction after the specified instruciton, or at the start of
+/* Inserts an instruction after the specified instruction, or at the start of
  * the basic block if the instruction is NULL. */
 void MVM_spesh_manipulate_insert_ins(MVMThreadContext *tc, MVMSpeshBB *bb, MVMSpeshIns *previous, MVMSpeshIns *to_insert) {
     /* Do the insertion. */
@@ -250,6 +271,17 @@ void MVM_spesh_manipulate_remove_handler_successors(MVMThreadContext *tc, MVMSpe
     bb->num_handler_succ = 0;
 }
 
+static void ensure_more_temps(MVMThreadContext *tc, MVMSpeshGraph *g) {
+    if (g->num_temps == g->alloc_temps) {
+        MVMSpeshTemporary *new_temps;
+        g->alloc_temps += 4;
+        new_temps = MVM_spesh_alloc(tc, g, g->alloc_temps * sizeof(MVMSpeshTemporary));
+        if (g->num_temps)
+            memcpy(new_temps, g->temps, g->num_temps * sizeof(MVMSpeshTemporary));
+        g->temps = new_temps;
+    }
+}
+
 /* Gets a temporary register of the specified kind to use in some transform.
  * Will only actually extend the frame if needed; if an existing temporary
  * was requested and then released, then it will just use a new version of
@@ -262,39 +294,40 @@ static void grow_facts(MVMThreadContext *tc, MVMSpeshGraph *g, MVMuint16 orig) {
     g->facts[orig] = new_fact_row;
     g->fact_counts[orig]++;
 }
-MVMSpeshOperand MVM_spesh_manipulate_get_temp_reg(MVMThreadContext *tc, MVMSpeshGraph *g, MVMuint16 kind) {
+static MVMSpeshOperand make_temp_reg(MVMThreadContext *tc, MVMSpeshGraph *g, MVMuint16 kind,
+        MVMuint16 reuse) {
     MVMSpeshOperand   result;
     MVMSpeshFacts   **new_facts;
     MVMuint16        *new_fact_counts;
     MVMuint16         i;
 
     /* First, see if we can find an existing free temporary; use it if so. */
-    for (i = 0; i < g->num_temps; i++) {
-        if (g->temps[i].kind == kind && !g->temps[i].in_use) {
-            /* Add new facts slot. */
-            MVMuint16 orig = g->temps[i].orig;
-            grow_facts(tc, g, orig);
+    if (reuse) {
+        for (i = 0; i < g->num_temps; i++) {
+            if (g->temps[i].kind == kind && !g->temps[i].in_use) {
+                /* Add new facts slot. */
+                MVMuint16 orig = g->temps[i].orig;
+                MVMSpeshFacts *new_fact_row = MVM_spesh_alloc(tc, g,
+                    (g->fact_counts[orig] + 1) * sizeof(MVMSpeshFacts));
+                memcpy(new_fact_row, g->facts[orig],
+                    g->fact_counts[orig] * sizeof(MVMSpeshFacts));
+                g->facts[orig] = new_fact_row;
+                g->fact_counts[orig]++;
 
-            /* Mark it in use and add extra version. */
-            g->temps[i].in_use++;
-            g->temps[i].i++;
+                /* Mark it in use and add extra version. */
+                g->temps[i].in_use++;
+                g->temps[i].i++;
 
-            /* Produce and return result. */
-            result.reg.orig = orig;
-            result.reg.i = g->temps[i].used_i = g->temps[i].i;
-            return result;
+                /* Produce and return result. */
+                result.reg.orig = orig;
+                result.reg.i = g->temps[i].used_i = g->temps[i].i;
+                return result;
+            }
         }
     }
 
     /* Make sure we've space in the temporaries store. */
-    if (g->num_temps == g->alloc_temps) {
-        MVMSpeshTemporary *new_temps;
-        g->alloc_temps += 4;
-        new_temps = MVM_spesh_alloc(tc, g, g->alloc_temps * sizeof(MVMSpeshTemporary));
-        if (g->num_temps)
-            memcpy(new_temps, g->temps, g->num_temps * sizeof(MVMSpeshTemporary));
-        g->temps = new_temps;
-    }
+    ensure_more_temps(tc, g);
 
     /* Allocate temporary and set up result. */
     g->temps[g->num_temps].orig   = result.reg.orig = g->num_locals;
@@ -329,6 +362,12 @@ MVMSpeshOperand MVM_spesh_manipulate_get_temp_reg(MVMThreadContext *tc, MVMSpesh
     return result;
 }
 
+/* Gets a temporary register, adding it to the set of registers of the
+ * frame. */
+MVMSpeshOperand MVM_spesh_manipulate_get_temp_reg(MVMThreadContext *tc, MVMSpeshGraph *g, MVMuint16 kind) {
+    return make_temp_reg(tc, g, kind, 1);
+}
+
 /* Releases a temporary register, so it can be used again later. */
 void MVM_spesh_manipulate_release_temp_reg(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshOperand temp) {
     MVMuint16 i;
@@ -405,6 +444,23 @@ MVMSpeshOperand MVM_spesh_manipulate_split_version(MVMThreadContext *tc, MVMSpes
     return new_version;
 }
 
+/* Gets a frame-unique register, adding it to the set of registers of the
+ * frame. This does not hand back a particular version, it just selects the
+ * unversioned register. */
+MVMuint16 MVM_spesh_manipulate_get_unique_reg(MVMThreadContext *tc, MVMSpeshGraph *g, MVMuint16 kind) {
+    return make_temp_reg(tc, g, kind, 0).reg.orig;
+}
+
+/* Get the current version of an SSA temporary. */
+MVMuint16 MVM_spesh_manipulate_get_current_version(MVMThreadContext *tc, MVMSpeshGraph *g,
+        MVMuint16 orig) {
+    MVMuint32 i;
+    for (i = 0; i < g->num_temps; i++)
+        if (g->temps[i].orig == orig)
+            return g->temps[i].i;
+    MVM_oops(tc, "Could not find register version for %d", orig);
+}
+
 MVMSpeshBB *MVM_spesh_manipulate_split_BB_at(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB *bb, MVMSpeshIns *ins) {
     MVMSpeshBB *new_bb = MVM_spesh_alloc(tc, g, sizeof(MVMSpeshBB));
     MVMSpeshBB *linear_next = bb->linear_next;
