@@ -140,7 +140,6 @@ static MVMint32 already_have_guard(MVMThreadContext *tc, MVMSpeshPluginGuardSet
 }
 
 static MVMSpeshPluginGuardSet * copy_guard_set(MVMThreadContext *tc, MVMSpeshPluginGuardSet *base_guards) {
-    MVMuint32 insert_pos, i;
     MVMSpeshPluginGuardSet *result = MVM_fixed_size_alloc(tc, tc->instance->fsa, sizeof(MVMSpeshPluginGuardSet));
     result->num_guards = (base_guards ? base_guards->num_guards : 0);
     result->guards = MVM_fixed_size_alloc(tc, tc->instance->fsa,
@@ -688,7 +687,7 @@ void MVM_spesh_plugin_rewrite_resolve(MVMThreadContext *tc, MVMSpeshGraph *g, MV
 
         /* Collect registers that go with each argument, and delete the arg
          * and prepargs instructions. */
-        MVMuint32 initial_arg_regs_length, arg_regs_length, i;
+        MVMuint32 arg_regs_length, i;
         MVMSpeshOperand *arg_regs = arg_ins_to_reg_list(tc, g, bb, ins, &arg_regs_length);
 
         /* Find result and add it to a spesh slot. */
@@ -707,7 +706,6 @@ void MVM_spesh_plugin_rewrite_resolve(MVMThreadContext *tc, MVMSpeshGraph *g, MV
         MVM_spesh_facts_object_facts(tc, g, ins->operands[0], resolvee);
 
         /* Prepend guards. */
-        initial_arg_regs_length = arg_regs_length;
         MVM_VECTOR_INIT(temps, 0);
         while (++guards_start <= guards_end) {
             MVMSpeshPluginGuard *guard = &(gs->guards[guards_start]);
