@@ -15,6 +15,10 @@ static void worker(MVMThreadContext *tc, MVMCallsite *callsite, MVMRegister *arg
             tc->instance->boot_types.BOOTArray);
     });
 
+#ifdef MVM_HAS_PTHREAD_SETNAME_NP
+    pthread_setname_np(pthread_self(), "spesh optimizer");
+#endif
+
     tc->instance->speshworker_thread_id = tc->thread_obj->body.thread_id;
 
     MVMROOT2(tc, updated_static_frames, previous_static_frames, {
@@ -54,7 +58,7 @@ static void worker(MVMThreadContext *tc, MVMCallsite *callsite, MVMRegister *arg
 
                     overview_data[0] = work_sequence_number;
                     overview_data[1] = now_time / 1000;
-                    overview_data[2] = (now_time - tc->instance->subscriptions.vm_startup_time) / 1000;
+                    overview_data[2] = (now_time - tc->instance->subscriptions.vm_startup_hrtime) / 1000;
                     overview_data[3] = (now_time - start_time) / 1000;
                 }
             }
