@@ -1,5 +1,21 @@
 #include "moar.h"
 #include "shiftjis_codeindex.h"
+static MVMint16 shift_jis_index_to_cp_array_offset (MVMThreadContext *tc, MVMint16 index) {
+    MVMuint16 offset = 0;
+    int i = 0;
+    if (index < 0 || SHIFTJIS_MAX_INDEX < index) return SHIFTJIS_NULL;
+    for (; i < SHIFTJIS_OFFSET_VALUES_ELEMS && shiftjis_offset_values[i].location < index; i++) {
+        if (index <= shiftjis_offset_values[i].location + shiftjis_offset_values[i].offset) {
+            return SHIFTJIS_NULL;
+        }
+        offset += shiftjis_offset_values[i].offset;
+    }
+    return index - offset;
+}
+static MVMGrapheme32 shift_jis_index_to_cp (MVMThreadContext *tc, MVMint16 index) {
+    MVMint16 offset = shift_jis_index_to_cp_array_offset(tc, index);
+    return offset == SHIFTJIS_NULL ? SHIFTJIS_NULL : shiftjis_index_to_cp_codepoints[offset];
+}
 /* Encodes the specified substring to ShiftJIS as specified here:
  * https://encoding.spec.whatwg.org/#shift_jis-decoder
  * The result string is NULL terminated, but the specified size is the non-null part. */
@@ -188,8 +204,8 @@ MVMString * MVM_string_shiftjis_decode(MVMThreadContext *tc,
         const MVMObject *result_type, char *windows125X_c, size_t num_bytes,
         MVMString *replacement, MVMint64 config) {
     MVMuint8 *bytes = (MVMuint8 *)windows125X_c;
-    MVMString *result = (MVMString *)REPR(result_type)->allocate(tc, STABLE(result_type));
-    size_t pos = 0, result_graphs, additional_bytes = 0;
+    MVMString *result;
+    size_t pos = 0, result_graphs;
     MVMStringIndex repl_length = replacement ? MVM_string_graphs(tc, replacement) : 0;
     MVMuint8 Shift_JIS_lead = 0x00;
     /* Stores a byte that we must run through the decoder a second time. Instead
@@ -201,9 +217,8 @@ MVMString * MVM_string_shiftjis_decode(MVMThreadContext *tc,
     int last_was_cr         = 0;
     MVMStringIndex result_size = num_bytes;
 
-    result->body.storage_type    = MVM_STRING_GRAPHEME_32;
     /* TODO allocate less? */
-    result->body.storage.blob_32 = MVM_malloc(sizeof(MVMGrapheme32) * result_size);
+    MVMGrapheme32 *buffer = MVM_malloc(sizeof(MVMGrapheme32) * result_size);
 
     result_graphs = 0;
     while (pos < num_bytes || repl_pos) {
@@ -244,6 +259,7 @@ MVMString * MVM_string_shiftjis_decode(MVMThreadContext *tc,
                     if (1 < repl_length) repl_pos++;
                 }
                 else {
+                    MVM_free(buffer);
                     /* Throw if it's unmapped */
                     MVM_exception_throw_adhoc(tc,
                         "Error decoding shiftjis string: could not decode byte 0x%hhX",
@@ -256,6 +272,7 @@ MVMString * MVM_string_shiftjis_decode(MVMThreadContext *tc,
                 continue;
             }
             else {
+                MVM_free(buffer);
                 MVM_exception_throw_adhoc(tc, "shiftjis decoder encountered an internal error.\n");
             }
         }
@@ -275,19 +292,21 @@ MVMString * MVM_string_shiftjis_decode(MVMThreadContext *tc,
         }
         if (result_graphs == result_size) {
             result_size += repl_length;
-            result->body.storage.blob_32 = MVM_realloc(result->body.storage.blob_32,
-                result_size * sizeof(MVMGrapheme32));
+            buffer = MVM_realloc(buffer, result_size * sizeof(MVMGrapheme32));
         }
-        result->body.storage.blob_32[result_graphs++] = graph;
+        buffer[result_graphs++] = graph;
     }
     /* If we end up with Shift_JIS_lead still set, that means we're missing a byte
      * that should have followed it. */
     if (Shift_JIS_lead != 0x00) {
+        MVM_free(buffer);
         MVM_exception_throw_adhoc(tc, "Error, ended decode of shiftjis expecting another byte. "
             "Last byte seen was 0x%hhX\n", Shift_JIS_lead);
     }
-    result->body.storage.blob_32 = MVM_realloc(result->body.storage.blob_32,
+    result = (MVMString *)REPR(result_type)->allocate(tc, STABLE(result_type));
+    result->body.storage.blob_32 = MVM_realloc(buffer,
         result_graphs * sizeof(MVMGrapheme32));
+    result->body.storage_type = MVM_STRING_GRAPHEME_32;
     result->body.num_graphs = result_graphs;
 
     return result;
@@ -295,10 +314,10 @@ MVMString * MVM_string_shiftjis_decode(MVMThreadContext *tc,
 /* Decodes using a decodestream. Decodes as far as it can with the input
  * buffers, or until a stopper is reached. */
 MVMuint32 MVM_string_shiftjis_decodestream(MVMThreadContext *tc, MVMDecodeStream *ds,
-                                         const MVMint32 *stopper_chars,
+                                         const MVMuint32 *stopper_chars,
                                          MVMDecodeStreamSeparators *seps) {
-    MVMint32 count = 0, total = 0;
-    MVMint32 bufsize;
+    MVMuint32 count = 0, total = 0;
+    MVMuint32 bufsize;
     MVMGrapheme32 *buffer = NULL;
     MVMDecodeStreamBytes *cur_bytes = NULL;
     MVMDecodeStreamBytes *last_accept_bytes = ds->bytes_head;
@@ -381,6 +400,7 @@ MVMuint32 MVM_string_shiftjis_decodestream(MVMThreadContext *tc, MVMDecodeStream
                     continue;
                 }
                 else {
+                    MVM_free(buffer);
                     MVM_exception_throw_adhoc(tc, "shiftjis decoder encountered an internal error. This bug should be reported.\n");
                 }
             }
