@@ -1,5 +1,7 @@
 #!/bin/sh
+export LC_ALL=C.UTF-8
 VERSION=$1
+[ ! "$VERSION" ] && exit 1
 {
     echo MANIFEST
     git ls-files | perl -ne "print unless /^3rdparty\/\w+$/"
@@ -8,7 +10,8 @@ VERSION=$1
         git ls-files | perl -pe "s{^}{$submod}"
         cd ../..;
     done
-} > MANIFEST
+} | sort > MANIFEST
 [ -d MoarVM-$VERSION ] || ln -s . MoarVM-$VERSION
-perl -pe "s{^}{MoarVM-$VERSION/}" MANIFEST | tar zc --owner=0 --group=0 --numeric-owner -T - -f MoarVM-$VERSION.tar.gz
+tag_timestamp=$(git log -1 --format='%at' $VERSION)
+perl -pe "s{^}{MoarVM-$VERSION/}" MANIFEST | tar c -H gnu --mode=go=rX,u+rw,a-s -I 'gzip -9n' --mtime=@$tag_timestamp --owner=0 --group=0 --numeric-owner -T - -f MoarVM-$VERSION.tar.gz
 rm MoarVM-$VERSION
